;-----PicAXE Plotter-----
; 
;Jan, 2012. Pieced together by Charles Wenzel from code by aobrien on the autohotkey.com forum.
;aobrien also credits Lexikos, krisky68, trenton_xavier, and other discussion threads
;
;The script is designed to enter serial data from a PicAXE into an openoffice.org spreadsheet.
;When openoffice.org starts for the first time, select "save as" and wait for a new file to be created.
;stop plotting with F3 and then save file as another name, if desired. Or, just kill the script.

#SingleInstance force ;drops old instance of program and starts a fresh one. Only one at a time will work.
SetTitleMatchMode, 2 ;looks for window title anywhere in string.

; Use Windows device manager to see which COM port your serial connection uses. Just observe the existing ports, then plug in
; your serial device from the PicAXE to see the new port. 

Gui, Add, Text,, File_Path:
Gui, Add, DropDownList, vRS232_Port, COM1|COM2|COM3||COM4|COM5|COM6|COM7|COM8|COM9|COM10|COM11|COM12|COM13|COM14
Gui, Add, DropDownList, vRS232_Baud, 110|300|600|2400|4800||9600|14400|19200|28800|38400|56000|57600|153600
Gui, Add, DropDownList, vRS232_Parity, Y|N||
Gui, Add, DropDownList, vRS232_Data, 7|8||
Gui, Add, DropDownList, vRS232_Stop, 0|1||2
Gui, Add, Edit, vFile_Path ym
Gui, Add, Button, default, OK  ; The label ButtonOK (if it exists) will be run when the button is pressed.
Gui, Show
return  ; End of auto-execute section. The script is idle until the user does something.
GuiClose:
ButtonOK:
Gui, Submit  ; Save the input from the user to each control's associated variable.

; Leaving File_Path blank will result in a file being located in the main C: directory, which is fine.

;F3 will close the COM port and exit the receive loop.

RS232_Settings   = %RS232_Port%:baud=%RS232_Baud% parity=%RS232_Parity% data=%RS232_Data% stop=%RS232_Stop% dtr=Off
Console_File_Name= %RS232_Port%_plotter.ods
Console_Title    = %Console_File_Name% - OpenOffice.org Calc
MsgBox, Requires openoffice.org Calc (spreadhseet). Use F3 to stop. Choose "save as" to start new spreadsheet. PicAXE sends number, then CR, or "^" to jump back to first cell,"<" to jump back to first column, or ">" to tab to next column.

;The following will check for a spreadsheet. If there isn't already one, open one and wait for user to select Save As.

IfWinNotExist, %Console_Title%
{
  Run, sCalc
  WinWait, Untitled 1 - OpenOffice.org Calc
  WinWait, Save As
  Clipboard = %File_Path%\%Console_File_Name%
  Send, ^v{ENTER}
  WinWaitClose 
} 
IfWinExist, %Console_Title%
{
WinActivate
}

;Quit_var is used to exit the RS232 COM port receive loop
;  0=Don't Exit; 1=Exit; Press F3 to set to 1 and exit script.
Quit_var = 0
;
RS232_FileHandle:=RS232_Initialize(RS232_Settings)

;RS232 receive loop

Loop
{
  Read_Data := RS232_Read(RS232_FileHandle,"0xFF",RS232_Bytes_Received)
  If (RS232_Bytes_Received > 0)
  {
  ;Prevent interruption during execution of this loop.
    Critical, On
      ;Begin Data to ASCII conversion
      ASCII =
      Read_Data_Num_Bytes := StrLen(Read_Data) / 2 ;RS232_Read() returns 2 characters for each byte

      Loop %Read_Data_Num_Bytes%
      {
        StringLeft, Byte, Read_Data, 2
        StringTrimLeft, Read_Data, Read_Data, 2
        Byte = 0x%Byte%
        Byte := Byte + 0 ;Convert to Decimal       
        ASCII_Chr := Chr(Byte)
        ASCII = %ASCII%%ASCII_Chr%
      }
            
      if ASCII = ^
        {
        ASCII = 
	SendInput ^{home}
        }
      else if ASCII = <
        {
        ASCII = 
        SendInput {home}
        }
      else if ASCII = >
        {
        ASCII = 
        SendInput {Tab}
        }
      ;these ifs look for three direction instructions, clears them so they don't print, and acts on them.
      SendRaw, %ASCII%

      ;End Data to ASCII conversion

    Critical, Off
    ;Sleep, 20
  }


 ;F3 sets Quit_var=1
  if Quit_var = 1
    Break
}
ExitApp

RS232_Close(RS232_FileHandle)

MsgBox, AHK is now disconnected from %RS232_Port%
;ExitApp ;Exit Script
Return


; RS232 COM Subroutine 

RS232_Initialize(RS232_Settings)
{
  ;###### Extract/Format the RS232 COM Port Number ######
  ;7/23/08 Thanks krisky68 for finding/solving the bug in which RS232 COM Ports greater than 9 didn't work.
  StringSplit, RS232_Temp, RS232_Settings, `:
  RS232_Temp1_Len := StrLen(RS232_Temp1)  ;For COM Ports > 9 \\.\ needs to prepended to the COM Port name.
  If (RS232_Temp1_Len > 4)                   ;So the valid names are
    RS232_COM = \\.\%RS232_Temp1%             ; ... COM8  COM9   \\.\COM10  \\.\COM11  \\.\COM12 and so on...
  Else                                          ;
    RS232_COM = %RS232_Temp1%

  ;8/10/09 A BIG Thanks to trenton_xavier for figuring out how to make COM Ports greater than 9 work for USB-Serial Dongles.
  StringTrimLeft, RS232_Settings, RS232_Settings, RS232_Temp1_Len+1 ;Remove the COM number (+1 for the semicolon) for BuildCommDCB.
  
  ;###### Build RS232 COM DCB ######
  ;Creates the structure that contains the RS232 COM Port number, baud rate,...
  VarSetCapacity(DCB, 28)
  BCD_Result := DllCall("BuildCommDCB"
       ,"str" , RS232_Settings ;lpDef
       ,"UInt", &DCB)        ;lpDCB
  If (BCD_Result <> 1)
  {
    MsgBox, There is a problem with Serial Port communication. `nFailed Dll BuildCommDCB, BCD_Result=%BCD_Result% `nThe Script Will Now Exit.
    Exit
  }

  ;###### Create RS232 COM File ######
  ;Creates the RS232 COM Port File Handle
  RS232_FileHandle := DllCall("CreateFile"
       ,"Str" , RS232_COM     ;File Name         
       ,"UInt", 0xC0000000   ;Desired Access
       ,"UInt", 3            ;Safe Mode
       ,"UInt", 0            ;Security Attributes
       ,"UInt", 3            ;Creation Disposition
       ,"UInt", 0            ;Flags And Attributes
       ,"UInt", 0            ;Template File
       ,"Cdecl Int")

  If (RS232_FileHandle < 1)
  {
    MsgBox, There is a problem with Serial Port communication. `nFailed Dll CreateFile, RS232_FileHandle=%RS232_FileHandle% `nThe Script Will Now Exit.
    Exit
  }

  ;###### Set COM State ######
  ;Sets the RS232 COM Port number, baud rate,...
  SCS_Result := DllCall("SetCommState"
       ,"UInt", RS232_FileHandle ;File Handle
       ,"UInt", &DCB)          ;Pointer to DCB structure
  If (SCS_Result <> 1)
  {
    MsgBox, There is a problem with Serial Port communication. `nFailed Dll SetCommState, SCS_Result=%SCS_Result% `nThe Script Will Now Exit.
    RS232_Close(RS232_FileHandle)
    Exit
  }

  ;###### Create the SetCommTimeouts Structure ######
  ReadIntervalTimeout        = 0xffffffff
  ReadTotalTimeoutMultiplier = 0x00000000
  ReadTotalTimeoutConstant   = 0x00000000
  WriteTotalTimeoutMultiplier= 0x00000000
  WriteTotalTimeoutConstant  = 0x00000000

  VarSetCapacity(Data, 20, 0) ; 5 * sizeof(DWORD)
  NumPut(ReadIntervalTimeout,         Data,  0, "UInt")
  NumPut(ReadTotalTimeoutMultiplier,  Data,  4, "UInt")
  NumPut(ReadTotalTimeoutConstant,    Data,  8, "UInt")
  NumPut(WriteTotalTimeoutMultiplier, Data, 12, "UInt")
  NumPut(WriteTotalTimeoutConstant,   Data, 16, "UInt")

  ;###### Set the RS232 COM Timeouts ######
  SCT_result := DllCall("SetCommTimeouts"
     ,"UInt", RS232_FileHandle ;File Handle
     ,"UInt", &Data)         ;Pointer to the data structure
  If (SCT_result <> 1)
  {
    MsgBox, There is a problem with Serial Port communication. `nFailed Dll SetCommState, SCT_result=%SCT_result% `nThe Script Will Now Exit.
    RS232_Close(RS232_FileHandle)
    Exit
  }
 
  Return %RS232_FileHandle%
}


;###### Close RS23 COM Subroutine ######

RS232_Close(RS232_FileHandle)
{
  ;###### Close the COM File ######
  CH_result := DllCall("CloseHandle", "UInt", RS232_FileHandle)
  If (CH_result <> 1)
    MsgBox, Failed Dll CloseHandle CH_result=%CH_result%

  Return
}


;###### Read from RS232 COM Subroutines ######

RS232_Read(RS232_FileHandle,Num_Bytes,ByRef RS232_Bytes_Received)
{
  SetFormat, Integer, HEX

  ;Set the Data buffer size, prefill with 0x55 = ASCII character "U"
  ;VarSetCapacity won't assign anything less than 3 bytes. Meaning: If you
  ;  tell it you want 1 or 2 byte size variable it will give you 3.
  Data_Length  := VarSetCapacity(Data, Num_Bytes, 0x55)
  
  ;###### Read the data from the RS232 COM Port ######
    Read_Result := DllCall("ReadFile"
       ,"UInt" , RS232_FileHandle   ; hFile
       ,"Str"  , Data             ; lpBuffer
       ,"Int"  , Num_Bytes        ; nNumberOfBytesToRead
       ,"UInt*", RS232_Bytes_Received   ; lpNumberOfBytesReceived
       ,"Int"  , 0)               ; lpOverlapped

  If (Read_Result <> 1)
  {
    MsgBox, There is a problem with Serial Port communication. `nFailed Dll ReadFile on RS232 COM, result=%Read_Result% - The Script Will Now Exit.
    RS232_Close(RS232_FileHandle)
    Exit
  }

  ;###### Format the received data ######
  ;This loop is necessary because AHK doesn't handle NULL (0x00) characters well.
  i = 0
  Data_HEX =
  Loop %RS232_Bytes_Received%
  {
    ;First byte into the Rx FIFO ends up at position 0

    Data_HEX_Temp := NumGet(Data, i, "UChar") ;Convert to HEX byte-by-byte
    StringTrimLeft, Data_HEX_Temp, Data_HEX_Temp, 2 ;Remove the 0x (added by the above line) from the front

    ;If there is only 1 character then add the leading "0'
    Length := StrLen(Data_HEX_Temp)
    If (Length =1)
      Data_HEX_Temp = 0%Data_HEX_Temp%

    i++

    ;Put it all together
    Data_HEX := Data_HEX . Data_HEX_Temp
  }
  
  SetFormat, Integer, DEC
  Data := Data_HEX

  Return Data
}

;########################################################################
;###### Exit Console Receive Loop #######################################
;########################################################################
F3::
Quit_var = 1
return